@@ -82,7 +82,6 @@ import datetime
 
 # Import Salt libs
 from salt.ext import six
-import salt.utils.boto3
 import salt.utils.compat
 import salt.utils.json
 import salt.utils.versions
@@ -182,7 +181,7 @@ def _find_apis_by_name(name, description=None,
             apis = _filter_apis_desc(description, apis)
         return {'restapi': [_convert_datetime_str(api) for api in apis]}
     except ClientError as e:
-        return {'error': salt.utils.boto3.get_error(e)}
+        return {'error': __utils__['boto3.get_error'](e)}
 
 
 def describe_apis(name=None, description=None, region=None, key=None, keyid=None, profile=None):
@@ -251,7 +250,7 @@ def create_api(name, description, cloneFrom=None,
         api = _convert_datetime_str(api)
         return {'created': True, 'restapi': api} if api else {'created': False}
     except ClientError as e:
-        return {'created': False, 'error': salt.utils.boto3.get_error(e)}
+        return {'created': False, 'error': __utils__['boto3.get_error'](e)}
 
 
 def delete_api(name, description=None, region=None, key=None, keyid=None, profile=None):
@@ -282,7 +281,7 @@ def delete_api(name, description=None, region=None, key=None, keyid=None, profil
         else:
             return {'deleted': False}
     except ClientError as e:
-        return {'deleted': False, 'error': salt.utils.boto3.get_error(e)}
+        return {'deleted': False, 'error': __utils__['boto3.get_error'](e)}
 
 
 def describe_api_resources(restApiId, region=None, key=None, keyid=None, profile=None):
@@ -303,7 +302,7 @@ def describe_api_resources(restApiId, region=None, key=None, keyid=None, profile
 
         return {'resources': resources}
     except ClientError as e:
-        return {'error': salt.utils.boto3.get_error(e)}
+        return {'error': __utils__['boto3.get_error'](e)}
 
 
 def describe_api_resource(restApiId, path,
@@ -364,7 +363,7 @@ def create_api_resources(restApiId, path,
         else:
             return {'created': False, 'error': 'unexpected error.'}
     except ClientError as e:
-        return {'created': False, 'error': salt.utils.boto3.get_error(e)}
+        return {'created': False, 'error': __utils__['boto3.get_error'](e)}
 
 
 def delete_api_resources(restApiId, path,
@@ -393,7 +392,7 @@ def delete_api_resources(restApiId, path,
         else:
             return {'deleted': False, 'error': 'no resource found by {0}'.format(path)}
     except ClientError as e:
-        return {'created': False, 'error': salt.utils.boto3.get_error(e)}
+        return {'created': False, 'error': __utils__['boto3.get_error'](e)}
 
 
 def describe_api_resource_method(restApiId, resourcePath, httpMethod,
@@ -421,7 +420,7 @@ def describe_api_resource_method(restApiId, resourcePath, httpMethod,
         method = conn.get_method(restApiId=restApiId, resourceId=resource['id'], httpMethod=httpMethod)
         return {'method': method}
     except ClientError as e:
-        return {'error': salt.utils.boto3.get_error(e)}
+        return {'error': __utils__['boto3.get_error'](e)}
 
 
 def describe_api_key(apiKey, region=None, key=None, keyid=None, profile=None):
@@ -440,7 +439,7 @@ def describe_api_key(apiKey, region=None, key=None, keyid=None, profile=None):
         response = conn.get_api_key(apiKey=apiKey)
         return {'apiKey': _convert_datetime_str(response)}
     except ClientError as e:
-        return {'error': salt.utils.boto3.get_error(e)}
+        return {'error': __utils__['boto3.get_error'](e)}
 
 
 def describe_api_keys(region=None, key=None, keyid=None, profile=None):
@@ -460,7 +459,7 @@ def describe_api_keys(region=None, key=None, keyid=None, profile=None):
 
         return {'apiKeys': [_convert_datetime_str(apikey) for apikey in apikeys]}
     except ClientError as e:
-        return {'error': salt.utils.boto3.get_error(e)}
+        return {'error': __utils__['boto3.get_error'](e)}
 
 
 def create_api_key(name, description, enabled=True, stageKeys=None,
@@ -498,7 +497,7 @@ def create_api_key(name, description, enabled=True, stageKeys=None,
 
         return {'created': True, 'apiKey': _convert_datetime_str(response)}
     except ClientError as e:
-        return {'created': False, 'error': salt.utils.boto3.get_error(e)}
+        return {'created': False, 'error': __utils__['boto3.get_error'](e)}
 
 
 def delete_api_key(apiKey, region=None, key=None, keyid=None, profile=None):
@@ -517,7 +516,7 @@ def delete_api_key(apiKey, region=None, key=None, keyid=None, profile=None):
         conn.delete_api_key(apiKey=apiKey)
         return {'deleted': True}
     except ClientError as e:
-        return {'deleted': False, 'error': salt.utils.boto3.get_error(e)}
+        return {'deleted': False, 'error': __utils__['boto3.get_error'](e)}
 
 
 def _api_key_patch_replace(conn, apiKey, path, value):
@@ -570,7 +569,7 @@ def update_api_key_description(apiKey, description, region=None, key=None, keyid
         response = _api_key_patch_replace(conn, apiKey, '/description', description)
         return {'updated': True, 'apiKey': _convert_datetime_str(response)}
     except ClientError as e:
-        return {'updated': False, 'error': salt.utils.boto3.get_error(e)}
+        return {'updated': False, 'error': __utils__['boto3.get_error'](e)}
 
 
 def enable_api_key(apiKey, region=None, key=None, keyid=None, profile=None):
@@ -589,7 +588,7 @@ def enable_api_key(apiKey, region=None, key=None, keyid=None, profile=None):
         response = _api_key_patch_replace(conn, apiKey, '/enabled', 'True')
         return {'apiKey': _convert_datetime_str(response)}
     except ClientError as e:
-        return {'error': salt.utils.boto3.get_error(e)}
+        return {'error': __utils__['boto3.get_error'](e)}
 
 
 def disable_api_key(apiKey, region=None, key=None, keyid=None, profile=None):
@@ -608,7 +607,7 @@ def disable_api_key(apiKey, region=None, key=None, keyid=None, profile=None):
         response = _api_key_patch_replace(conn, apiKey, '/enabled', 'False')
         return {'apiKey': _convert_datetime_str(response)}
     except ClientError as e:
-        return {'error': salt.utils.boto3.get_error(e)}
+        return {'error': __utils__['boto3.get_error'](e)}
 
 
 def associate_api_key_stagekeys(apiKey, stagekeyslist, region=None, key=None, keyid=None, profile=None):
@@ -629,7 +628,7 @@ def associate_api_key_stagekeys(apiKey, stagekeyslist, region=None, key=None, ke
         response = _api_key_patch_add(conn, apiKey, pvlist)
         return {'associated': True, 'apiKey': _convert_datetime_str(response)}
     except ClientError as e:
-        return {'associated': False, 'error': salt.utils.boto3.get_error(e)}
+        return {'associated': False, 'error': __utils__['boto3.get_error'](e)}
 
 
 def disassociate_api_key_stagekeys(apiKey, stagekeyslist, region=None, key=None, keyid=None, profile=None):
@@ -650,7 +649,7 @@ def disassociate_api_key_stagekeys(apiKey, stagekeyslist, region=None, key=None,
         response = _api_key_patch_remove(conn, apiKey, pvlist)
         return {'disassociated': True}
     except ClientError as e:
-        return {'disassociated': False, 'error': salt.utils.boto3.get_error(e)}
+        return {'disassociated': False, 'error': __utils__['boto3.get_error'](e)}
 
 
 def describe_api_deployments(restApiId, region=None, key=None, keyid=None, profile=None):
@@ -678,7 +677,7 @@ def describe_api_deployments(restApiId, region=None, key=None, keyid=None, profi
 
         return {'deployments': [_convert_datetime_str(deployment) for deployment in deployments]}
     except ClientError as e:
-        return {'error': salt.utils.boto3.get_error(e)}
+        return {'error': __utils__['boto3.get_error'](e)}
 
 
 def describe_api_deployment(restApiId, deploymentId, region=None, key=None, keyid=None, profile=None):
@@ -697,7 +696,7 @@ def describe_api_deployment(restApiId, deploymentId, region=None, key=None, keyi
         deployment = conn.get_deployment(restApiId=restApiId, deploymentId=deploymentId)
         return {'deployment': _convert_datetime_str(deployment)}
     except ClientError as e:
-        return {'error': salt.utils.boto3.get_error(e)}
+        return {'error': __utils__['boto3.get_error'](e)}
 
 
 def activate_api_deployment(restApiId, stageName, deploymentId,
@@ -720,7 +719,7 @@ def activate_api_deployment(restApiId, stageName, deploymentId,
                                                        'value': deploymentId}])
         return {'set': True, 'response': _convert_datetime_str(response)}
     except ClientError as e:
-        return {'set': False, 'error': salt.utils.boto3.get_error(e)}
+        return {'set': False, 'error': __utils__['boto3.get_error'](e)}
 
 
 def create_api_deployment(restApiId, stageName, stageDescription='', description='', cacheClusterEnabled=False,
@@ -747,7 +746,7 @@ def create_api_deployment(restApiId, stageName, stageDescription='', description
                                             variables=variables)
         return {'created': True, 'deployment': _convert_datetime_str(deployment)}
     except ClientError as e:
-        return {'created': False, 'error': salt.utils.boto3.get_error(e)}
+        return {'created': False, 'error': __utils__['boto3.get_error'](e)}
 
 
 def delete_api_deployment(restApiId, deploymentId, region=None, key=None, keyid=None, profile=None):
@@ -766,7 +765,7 @@ def delete_api_deployment(restApiId, deploymentId, region=None, key=None, keyid=
         conn.delete_deployment(restApiId=restApiId, deploymentId=deploymentId)
         return {'deleted': True}
     except ClientError as e:
-        return {'deleted': False, 'error': salt.utils.boto3.get_error(e)}
+        return {'deleted': False, 'error': __utils__['boto3.get_error'](e)}
 
 
 def overwrite_api_stage_variables(restApiId, stageName, variables, region=None, key=None, keyid=None, profile=None):
@@ -812,7 +811,7 @@ def overwrite_api_stage_variables(restApiId, stageName, variables, region=None,
 
         return {'overwrite': True, 'stage': _convert_datetime_str(stage)}
     except ClientError as e:
-        return {'overwrite': False, 'error': salt.utils.boto3.get_error(e)}
+        return {'overwrite': False, 'error': __utils__['boto3.get_error'](e)}
 
 
 def describe_api_stage(restApiId, stageName, region=None, key=None, keyid=None, profile=None):
@@ -831,7 +830,7 @@ def describe_api_stage(restApiId, stageName, region=None, key=None, keyid=None,
         stage = conn.get_stage(restApiId=restApiId, stageName=stageName)
         return {'stage': _convert_datetime_str(stage)}
     except ClientError as e:
-        return {'error': salt.utils.boto3.get_error(e)}
+        return {'error': __utils__['boto3.get_error'](e)}
 
 
 def describe_api_stages(restApiId, deploymentId, region=None, key=None, keyid=None, profile=None):
@@ -850,7 +849,7 @@ def describe_api_stages(restApiId, deploymentId, region=None, key=None, keyid=No
         stages = conn.get_stages(restApiId=restApiId, deploymentId=deploymentId)
         return {'stages': [_convert_datetime_str(stage) for stage in stages['item']]}
     except ClientError as e:
-        return {'error': salt.utils.boto3.get_error(e)}
+        return {'error': __utils__['boto3.get_error'](e)}
 
 
 def create_api_stage(restApiId, stageName, deploymentId, description='',
@@ -876,7 +875,7 @@ def create_api_stage(restApiId, stageName, deploymentId, description='',
                                   cacheClusterSize=cacheClusterSize, variables=variables)
         return {'created': True, 'stage': _convert_datetime_str(stage)}
     except ClientError as e:
-        return {'created': False, 'error': salt.utils.boto3.get_error(e)}
+        return {'created': False, 'error': __utils__['boto3.get_error'](e)}
 
 
 def delete_api_stage(restApiId, stageName, region=None, key=None, keyid=None, profile=None):
@@ -895,7 +894,7 @@ def delete_api_stage(restApiId, stageName, region=None, key=None, keyid=None, pr
         conn.delete_stage(restApiId=restApiId, stageName=stageName)
         return {'deleted': True}
     except ClientError as e:
-        return {'deleted': False, 'error': salt.utils.boto3.get_error(e)}
+        return {'deleted': False, 'error': __utils__['boto3.get_error'](e)}
 
 
 def flush_api_stage_cache(restApiId, stageName, region=None, key=None, keyid=None, profile=None):
@@ -914,7 +913,7 @@ def flush_api_stage_cache(restApiId, stageName, region=None, key=None, keyid=Non
         conn.flush_stage_cache(restApiId=restApiId, stageName=stageName)
         return {'flushed': True}
     except ClientError as e:
-        return {'flushed': False, 'error': salt.utils.boto3.get_error(e)}
+        return {'flushed': False, 'error': __utils__['boto3.get_error'](e)}
 
 
 def create_api_method(restApiId, resourcePath, httpMethod, authorizationType,
@@ -946,7 +945,7 @@ def create_api_method(restApiId, resourcePath, httpMethod, authorizationType,
         return {'created': False, 'error': 'Failed to create method'}
 
     except ClientError as e:
-        return {'created': False, 'error': salt.utils.boto3.get_error(e)}
+        return {'created': False, 'error': __utils__['boto3.get_error'](e)}
 
 
 def describe_api_method(restApiId, resourcePath, httpMethod, region=None, key=None, keyid=None, profile=None):
@@ -969,7 +968,7 @@ def describe_api_method(restApiId, resourcePath, httpMethod, region=None, key=No
             return {'method': _convert_datetime_str(method)}
         return {'error': 'get API method failed: no such resource'}
     except ClientError as e:
-        return {'error': salt.utils.boto3.get_error(e)}
+        return {'error': __utils__['boto3.get_error'](e)}
 
 
 def delete_api_method(restApiId, resourcePath, httpMethod, region=None, key=None, keyid=None, profile=None):
@@ -992,7 +991,7 @@ def delete_api_method(restApiId, resourcePath, httpMethod, region=None, key=None
             return {'deleted': True}
         return {'deleted': False, 'error': 'get API method failed: no such resource'}
     except ClientError as e:
-        return {'deleted': False, 'error': salt.utils.boto3.get_error(e)}
+        return {'deleted': False, 'error': __utils__['boto3.get_error'](e)}
 
 
 def create_api_method_response(restApiId, resourcePath, httpMethod, statusCode, responseParameters=None,
@@ -1022,7 +1021,7 @@ def create_api_method_response(restApiId, resourcePath, httpMethod, statusCode,
             return {'created': True, 'response': response}
         return {'created': False, 'error': 'no such resource'}
     except ClientError as e:
-        return {'created': False, 'error': salt.utils.boto3.get_error(e)}
+        return {'created': False, 'error': __utils__['boto3.get_error'](e)}
 
 
 def delete_api_method_response(restApiId, resourcePath, httpMethod, statusCode,
@@ -1047,7 +1046,7 @@ def delete_api_method_response(restApiId, resourcePath, httpMethod, statusCode,
             return {'deleted': True}
         return {'deleted': False, 'error': 'no such resource'}
     except ClientError as e:
-        return {'deleted': False, 'error': salt.utils.boto3.get_error(e)}
+        return {'deleted': False, 'error': __utils__['boto3.get_error'](e)}
 
 
 def describe_api_method_response(restApiId, resourcePath, httpMethod, statusCode,
@@ -1072,7 +1071,7 @@ def describe_api_method_response(restApiId, resourcePath, httpMethod, statusCode
             return {'response': _convert_datetime_str(response)}
         return {'error': 'no such resource'}
     except ClientError as e:
-        return {'error': salt.utils.boto3.get_error(e)}
+        return {'error': __utils__['boto3.get_error'](e)}
 
 
 def describe_api_models(restApiId, region=None, key=None, keyid=None, profile=None):
@@ -1091,7 +1090,7 @@ def describe_api_models(restApiId, region=None, key=None, keyid=None, profile=No
         models = _multi_call(conn.get_models, 'items', restApiId=restApiId)
         return {'models': [_convert_datetime_str(model) for model in models]}
     except ClientError as e:
-        return {'error': salt.utils.boto3.get_error(e)}
+        return {'error': __utils__['boto3.get_error'](e)}
 
 
 def describe_api_model(restApiId, modelName, flatten=True, region=None, key=None, keyid=None, profile=None):
@@ -1110,7 +1109,7 @@ def describe_api_model(restApiId, modelName, flatten=True, region=None, key=None
         model = conn.get_model(restApiId=restApiId, modelName=modelName, flatten=flatten)
         return {'model': _convert_datetime_str(model)}
     except ClientError as e:
-        return {'error': salt.utils.boto3.get_error(e)}
+        return {'error': __utils__['boto3.get_error'](e)}
 
 
 def api_model_exists(restApiId, modelName, region=None, key=None, keyid=None, profile=None):
@@ -1154,7 +1153,7 @@ def update_api_model_schema(restApiId, modelName, schema, region=None, key=None,
         response = _api_model_patch_replace(conn, restApiId, modelName, '/schema', schema_json)
         return {'updated': True, 'model': _convert_datetime_str(response)}
     except ClientError as e:
-        return {'updated': False, 'error': salt.utils.boto3.get_error(e)}
+        return {'updated': False, 'error': __utils__['boto3.get_error'](e)}
 
 
 def delete_api_model(restApiId, modelName, region=None, key=None, keyid=None, profile=None):
@@ -1173,7 +1172,7 @@ def delete_api_model(restApiId, modelName, region=None, key=None, keyid=None, pr
         conn.delete_model(restApiId=restApiId, modelName=modelName)
         return {'deleted': True}
     except ClientError as e:
-        return {'deleted': False, 'error': salt.utils.boto3.get_error(e)}
+        return {'deleted': False, 'error': __utils__['boto3.get_error'](e)}
 
 
 def create_api_model(restApiId, modelName, modelDescription, schema, contentType='application/json',
@@ -1196,7 +1195,7 @@ def create_api_model(restApiId, modelName, modelDescription, schema, contentType
                                   schema=schema_json, contentType=contentType)
         return {'created': True, 'model': _convert_datetime_str(model)}
     except ClientError as e:
-        return {'created': False, 'error': salt.utils.boto3.get_error(e)}
+        return {'created': False, 'error': __utils__['boto3.get_error'](e)}
 
 
 def describe_api_integration(restApiId, resourcePath, httpMethod, region=None, key=None, keyid=None, profile=None):
@@ -1219,7 +1218,7 @@ def describe_api_integration(restApiId, resourcePath, httpMethod, region=None, k
             return {'integration': _convert_datetime_str(integration)}
         return {'error': 'no such resource'}
     except ClientError as e:
-        return {'error': salt.utils.boto3.get_error(e)}
+        return {'error': __utils__['boto3.get_error'](e)}
 
 
 def describe_api_integration_response(restApiId, resourcePath, httpMethod, statusCode,
@@ -1244,7 +1243,7 @@ def describe_api_integration_response(restApiId, resourcePath, httpMethod, statu
             return {'response': _convert_datetime_str(response)}
         return {'error': 'no such resource'}
     except ClientError as e:
-        return {'error': salt.utils.boto3.get_error(e)}
+        return {'error': __utils__['boto3.get_error'](e)}
 
 
 def delete_api_integration(restApiId, resourcePath, httpMethod, region=None, key=None, keyid=None, profile=None):
@@ -1267,7 +1266,7 @@ def delete_api_integration(restApiId, resourcePath, httpMethod, region=None, key
             return {'deleted': True}
         return {'deleted': False, 'error': 'no such resource'}
     except ClientError as e:
-        return {'deleted': False, 'error': salt.utils.boto3.get_error(e)}
+        return {'deleted': False, 'error': __utils__['boto3.get_error'](e)}
 
 
 def delete_api_integration_response(restApiId, resourcePath, httpMethod, statusCode,
@@ -1292,7 +1291,7 @@ def delete_api_integration_response(restApiId, resourcePath, httpMethod, statusC
             return {'deleted': True}
         return {'deleted': False, 'error': 'no such resource'}
     except ClientError as e:
-        return {'deleted': False, 'error': salt.utils.boto3.get_error(e)}
+        return {'deleted': False, 'error': __utils__['boto3.get_error'](e)}
 
 
 def _get_role_arn(name, region=None, key=None, keyid=None, profile=None):
@@ -1349,7 +1348,7 @@ def create_api_integration(restApiId, resourcePath, httpMethod, integrationType,
             return {'created': True, 'integration': integration}
         return {'created': False, 'error': 'no such resource'}
     except ClientError as e:
-        return {'created': False, 'error': salt.utils.boto3.get_error(e)}
+        return {'created': False, 'error': __utils__['boto3.get_error'](e)}
 
 
 def create_api_integration_response(restApiId, resourcePath, httpMethod, statusCode, selectionPattern,
@@ -1382,7 +1381,7 @@ def create_api_integration_response(restApiId, resourcePath, httpMethod, statusC
             return {'created': True, 'response': response}
         return {'created': False, 'error': 'no such resource'}
     except ClientError as e:
-        return {'created': False, 'error': salt.utils.boto3.get_error(e)}
+        return {'created': False, 'error': __utils__['boto3.get_error'](e)}
 
 
 def _filter_plans(attr, name, plans):
@@ -1418,7 +1417,7 @@ def describe_usage_plans(name=None, plan_id=None, region=None, key=None, keyid=N
         return {'plans': [_convert_datetime_str(plan) for plan in plans]}
 
     except ClientError as e:
-        return {'error': salt.utils.boto3.get_error(e)}
+        return {'error': __utils__['boto3.get_error'](e)}
 
 
 def _validate_throttle(throttle):
@@ -1497,7 +1496,7 @@ def create_usage_plan(name, description=None, throttle=None, quota=None, region=
         res = conn.create_usage_plan(**values)
         return {'created': True, 'result': res}
     except ClientError as e:
-        return {'error': salt.utils.boto3.get_error(e)}
+        return {'error': __utils__['boto3.get_error'](e)}
     except (TypeError, ValueError) as e:
         return {'error': six.text_type(e)}
 
@@ -1571,7 +1570,7 @@ def update_usage_plan(plan_id, throttle=None, quota=None, region=None, key=None,
         return {'updated': False}
 
     except ClientError as e:
-        return {'error': salt.utils.boto3.get_error(e)}
+        return {'error': __utils__['boto3.get_error'](e)}
     except (TypeError, ValueError) as e:
         return {'error': six.text_type(e)}
 
@@ -1600,7 +1599,7 @@ def delete_usage_plan(plan_id, region=None, key=None, keyid=None, profile=None):
             res = conn.delete_usage_plan(usagePlanId=plan_id)
         return {'deleted': True, 'usagePlanId': plan_id}
     except ClientError as e:
-        return {'error': salt.utils.boto3.get_error(e)}
+        return {'error': __utils__['boto3.get_error'](e)}
 
 
 def _update_usage_plan_apis(plan_id, apis, op, region=None, key=None, keyid=None, profile=None):
@@ -1634,7 +1633,7 @@ def _update_usage_plan_apis(plan_id, apis, op, region=None, key=None, keyid=None
                                          patchOperations=patchOperations)
         return {'success': True, 'result': res}
     except ClientError as e:
-        return {'error': salt.utils.boto3.get_error(e)}
+        return {'error': __utils__['boto3.get_error'](e)}
     except Exception as e:
         return {'error': e}
 
