@@ -202,19 +202,17 @@ import tornado.ioloop
 import tornado.web
 import tornado.gen
 from tornado.concurrent import Future
-from zmq.eventloop import ioloop
-from salt.ext import six
 # pylint: enable=import-error
 
-# instantiate the zmq IOLoop (specialized poller)
-ioloop.install()
-
 # salt imports
+import salt.ext.six as six
 import salt.netapi
 import salt.utils.args
 import salt.utils.event
+import salt.utils.jid
 import salt.utils.json
 import salt.utils.yaml
+import salt.utils.zeromq
 from salt.utils.event import tagify
 import salt.client
 import salt.runner
@@ -225,6 +223,7 @@ from salt.exceptions import (
     EauthAuthenticationError
 )
 
+salt.utils.zeromq.install_zmq()
 json = salt.utils.json.import_json()
 log = logging.getLogger(__name__)
 
@@ -245,7 +244,7 @@ def _json_dumps(obj, **kwargs):
 
 # # master side
 #  - "runner" (done)
-#  - "wheel" (need async api...)
+#  - "wheel" (need asynchronous api...)
 
 
 AUTH_TOKEN_HEADER = 'X-Auth-Token'
@@ -274,7 +273,7 @@ class Any(Future):
 class EventListener(object):
     '''
     Class responsible for listening to the salt master event bus and updating
-    futures. This is the core of what makes this async, this allows us to do
+    futures. This is the core of what makes this asynchronous, this allows us to do
     non-blocking work in the main processes and "wait" for an event to happen
     '''
 
@@ -301,15 +300,15 @@ class EventListener(object):
 
         self.event.set_event_handler(self._handle_event_socket_recv)
 
-    def clean_timeout_futures(self, request):
+    def clean_by_request(self, request):
         '''
         Remove all futures that were waiting for request `request` since it is done waiting
         '''
         if request not in self.request_map:
             return
-        for tag, future in self.request_map[request]:
+        for tag, matcher, future in self.request_map[request]:
             # timeout the future
-            self._timeout_future(tag, future)
+            self._timeout_future(tag, matcher, future)
             # remove the timeout
             if future in self.timeout_map:
                 tornado.ioloop.IOLoop.current().remove_timeout(self.timeout_map[future])
@@ -317,14 +316,27 @@ class EventListener(object):
 
         del self.request_map[request]
 
+    @staticmethod
+    def prefix_matcher(mtag, tag):
+        if mtag is None or tag is None:
+            raise TypeError('mtag or tag can not be None')
+        return mtag.startswith(tag)
+
+    @staticmethod
+    def exact_matcher(mtag, tag):
+        if mtag is None or tag is None:
+            raise TypeError('mtag or tag can not be None')
+        return mtag == tag
+
     def get_event(self,
                   request,
                   tag='',
+                  matcher=prefix_matcher.__func__,
                   callback=None,
                   timeout=None
                   ):
         '''
-        Get an event (async of course) return a future that will get it later
+        Get an event (asynchronous of course) return a future that will get it later
         '''
         # if the request finished, no reason to allow event fetching, since we
         # can't send back to the client
@@ -339,43 +351,52 @@ class EventListener(object):
                 tornado.ioloop.IOLoop.current().add_callback(callback, future)
             future.add_done_callback(handle_future)
         # add this tag and future to the callbacks
-        self.tag_map[tag].append(future)
-        self.request_map[request].append((tag, future))
+        self.tag_map[(tag, matcher)].append(future)
+        self.request_map[request].append((tag, matcher, future))
 
         if timeout:
-            timeout_future = tornado.ioloop.IOLoop.current().call_later(timeout, self._timeout_future, tag, future)
+            timeout_future = tornado.ioloop.IOLoop.current().call_later(timeout, self._timeout_future, tag, matcher, future)
             self.timeout_map[future] = timeout_future
 
         return future
 
-    def _timeout_future(self, tag, future):
+    def _timeout_future(self, tag, matcher, future):
         '''
         Timeout a specific future
         '''
-        if tag not in self.tag_map:
+        if (tag, matcher) not in self.tag_map:
             return
         if not future.done():
             future.set_exception(TimeoutException())
-            self.tag_map[tag].remove(future)
-        if len(self.tag_map[tag]) == 0:
-            del self.tag_map[tag]
+            self.tag_map[(tag, matcher)].remove(future)
+        if len(self.tag_map[(tag, matcher)]) == 0:
+            del self.tag_map[(tag, matcher)]
 
     def _handle_event_socket_recv(self, raw):
         '''
         Callback for events on the event sub socket
         '''
         mtag, data = self.event.unpack(raw, self.event.serial)
+
         # see if we have any futures that need this info:
-        for tag_prefix, futures in six.iteritems(self.tag_map):
-            if mtag.startswith(tag_prefix):
-                for future in futures:
-                    if future.done():
-                        continue
-                    future.set_result({'data': data, 'tag': mtag})
-                    self.tag_map[tag_prefix].remove(future)
-                    if future in self.timeout_map:
-                        tornado.ioloop.IOLoop.current().remove_timeout(self.timeout_map[future])
-                        del self.timeout_map[future]
+        for (tag, matcher), futures in six.iteritems(self.tag_map):
+            try:
+                is_matched = matcher(mtag, tag)
+            except Exception:
+                log.error('Failed to run a matcher.', exc_info=True)
+                is_matched = False
+
+            if not is_matched:
+                continue
+
+            for future in futures:
+                if future.done():
+                    continue
+                future.set_result({'data': data, 'tag': mtag})
+                self.tag_map[(tag, matcher)].remove(future)
+                if future in self.timeout_map:
+                    tornado.ioloop.IOLoop.current().remove_timeout(self.timeout_map[future])
+                    del self.timeout_map[future]
 
 
 class BaseSaltAPIHandler(tornado.web.RequestHandler):  # pylint: disable=W0223
@@ -416,6 +437,9 @@ class BaseSaltAPIHandler(tornado.web.RequestHandler):  # pylint: disable=W0223
                 'runner_async': None,  # empty, since we use the same client as `runner`
                 }
 
+        if not hasattr(self, 'ckminions'):
+            self.ckminions = salt.utils.minions.CkMinions(self.application.opts)
+
     @property
     def token(self):
         '''
@@ -471,7 +495,7 @@ class BaseSaltAPIHandler(tornado.web.RequestHandler):  # pylint: disable=W0223
         timeout a session
         '''
         # TODO: set a header or something??? so we know it was a timeout
-        self.application.event_listener.clean_timeout_futures(self)
+        self.application.event_listener.clean_by_request(self)
 
     def on_finish(self):
         '''
@@ -629,7 +653,7 @@ class SaltAuthHandler(BaseSaltAPIHandler):  # pylint: disable=W0223
 
         self.write(self.serialize(ret))
 
-    # TODO: make async? Underlying library isn't... and we ARE making disk calls :(
+    # TODO: make asynchronous? Underlying library isn't... and we ARE making disk calls :(
     def post(self):
         '''
         :ref:`Authenticate  <rest_tornado-auth>` against Salt's eauth system
@@ -906,7 +930,14 @@ class SaltAPIHandler(BaseSaltAPIHandler):  # pylint: disable=W0223
         '''
         Dispatch local client commands
         '''
-        chunk_ret = {}
+        # Generate jid before triggering a job to subscribe all returns from minions
+        chunk['jid'] = salt.utils.jid.gen_jid(self.application.opts)
+
+        # start listening for the event before we fire the job to avoid races
+        events = [
+            self.application.event_listener.get_event(self, tag='salt/job/'+chunk['jid']),
+            self.application.event_listener.get_event(self, tag='syndic/job/'+chunk['jid']),
+        ]
 
         f_call = self._format_call_run_job_async(chunk)
         # fire a job off
@@ -915,80 +946,96 @@ class SaltAPIHandler(BaseSaltAPIHandler):  # pylint: disable=W0223
         # if the job didn't publish, lets not wait around for nothing
         # TODO: set header??
         if 'jid' not in pub_data:
+            for future in events:
+                try:
+                    future.set_result(None)
+                except Exception:
+                    pass
             raise tornado.gen.Return('No minions matched the target. No command was sent, no jid was assigned.')
 
-        # seed minions_remaining with the pub_data
-        minions_remaining = pub_data['minions']
+        # Map of minion_id -> returned for all minions we think we need to wait on
+        minions = {m: False for m in pub_data['minions']}
 
-        syndic_min_wait = None
+        # minimum time required for return to complete. By default no waiting, if
+        # we are a syndic then we must wait syndic_wait at a minimum
+        min_wait_time = Future()
+        min_wait_time.set_result(True)
+
+        # wait syndic a while to avoid missing published events
         if self.application.opts['order_masters']:
-            syndic_min_wait = tornado.gen.sleep(self.application.opts['syndic_wait'])
+            min_wait_time = tornado.gen.sleep(self.application.opts['syndic_wait'])
 
-        job_not_running = self.job_not_running(pub_data['jid'],
-                                               chunk['tgt'],
-                                               f_call['kwargs']['tgt_type'],
-                                               minions_remaining=minions_remaining
-                                               )
+        # To ensure job_not_running and all_return are terminated by each other, communicate using a future
+        is_finished = tornado.gen.sleep(self.application.opts['gather_job_timeout'])
 
-        # if we have a min_wait, do that
-        if syndic_min_wait is not None:
-            yield syndic_min_wait
-        # we are completed when either all minions return or the job isn't running anywhere
-        chunk_ret = yield self.all_returns(pub_data['jid'],
-                                           finish_futures=[job_not_running],
-                                           minions_remaining=minions_remaining,
-                                           )
+        # ping until the job is not running, while doing so, if we see new minions returning
+        # that they are running the job, add them to the list
+        tornado.ioloop.IOLoop.current().spawn_callback(self.job_not_running, pub_data['jid'],
+                                                      chunk['tgt'],
+                                                      f_call['kwargs']['tgt_type'],
+                                                      minions,
+                                                      is_finished)
 
-        raise tornado.gen.Return(chunk_ret)
+        def more_todo():
+            '''Check if there are any more minions we are waiting on returns from
+            '''
+            return any(x is False for x in six.itervalues(minions))
 
-    @tornado.gen.coroutine
-    def all_returns(self,
-                    jid,
-                    finish_futures=None,
-                    minions_remaining=None,
-                    ):
-        '''
-        Return a future which will complete once all returns are completed
-        (according to minions_remaining), or one of the passed in "finish_futures" completes
-        '''
-        if finish_futures is None:
-            finish_futures = []
-        if minions_remaining is None:
-            minions_remaining = []
-
-        ret_tag = tagify([jid, 'ret'], 'job')
+        # here we want to follow the behavior of LocalClient.get_iter_returns
+        # namely we want to wait at least syndic_wait (assuming we are a syndic)
+        # and that there are no more jobs running on minions. We are allowed to exit
+        # early if gather_job_timeout has been exceeded
         chunk_ret = {}
         while True:
-            ret_event = self.application.event_listener.get_event(self,
-                                                      tag=ret_tag,
-                                                      )
-            f = yield Any([ret_event] + finish_futures)
-            if f in finish_futures:
-                raise tornado.gen.Return(chunk_ret)
-            event = f.result()
-            chunk_ret[event['data']['id']] = event['data']['return']
-            # its possible to get a return that wasn't in the minion_remaining list
+            to_wait = events+[is_finished]
+            if not min_wait_time.done():
+                to_wait += [min_wait_time]
+
+            def cancel_inflight_futures():
+                for event in to_wait:
+                    if not event.done():
+                        event.set_result(None)
+            f = yield Any(to_wait)
             try:
-                minions_remaining.remove(event['data']['id'])
-            except ValueError:
+                # When finished entire routine, cleanup other futures and return result
+                if f is is_finished:
+                    cancel_inflight_futures()
+                    raise tornado.gen.Return(chunk_ret)
+                elif f is min_wait_time:
+                    if not more_todo():
+                        cancel_inflight_futures()
+                        raise tornado.gen.Return(chunk_ret)
+                    continue
+                f_result = f.result()
+                # if this is a start, then we need to add it to the pile
+                if f_result['tag'].endswith('/new'):
+                    for minion_id in f_result['data']['minions']:
+                        if minion_id not in minions:
+                            minions[minion_id] = False
+                else:
+                    chunk_ret[f_result['data']['id']] = f_result['data']['return']
+                    # clear finished event future
+                    minions[f_result['data']['id']] = True
+
+                    # if there are no more minions to wait for, then we are done
+                    if not more_todo() and min_wait_time.done():
+                        cancel_inflight_futures()
+                        raise tornado.gen.Return(chunk_ret)
+
+            except TimeoutException:
                 pass
-            if len(minions_remaining) == 0:
-                raise tornado.gen.Return(chunk_ret)
+
+            if f == events[0]:
+                events[0] = self.application.event_listener.get_event(self, tag='salt/job/'+chunk['jid'])
+            else:
+                events[1] = self.application.event_listener.get_event(self, tag='syndic/job/'+chunk['jid'])
 
     @tornado.gen.coroutine
-    def job_not_running(self,
-                  jid,
-                  tgt,
-                  tgt_type,
-                  minions_remaining=None,
-                  ):
+    def job_not_running(self, jid, tgt, tgt_type, minions, is_finished):
         '''
         Return a future which will complete once jid (passed in) is no longer
         running on tgt
         '''
-        if minions_remaining is None:
-            minions_remaining = []
-
         ping_pub_data = yield self.saltclients['local'](tgt,
                                                         'saltutil.find_job',
                                                         [jid],
@@ -998,10 +1045,16 @@ class SaltAPIHandler(BaseSaltAPIHandler):  # pylint: disable=W0223
         minion_running = False
         while True:
             try:
-                event = yield self.application.event_listener.get_event(self,
-                                                                        tag=ping_tag,
-                                                                        timeout=self.application.opts['gather_job_timeout'],
-                                                                        )
+                event = self.application.event_listener.get_event(self,
+                                                                  tag=ping_tag,
+                                                                  timeout=self.application.opts['gather_job_timeout'])
+                f = yield Any([event, is_finished])
+                # When finished entire routine, cleanup other futures and return result
+                if f is is_finished:
+                    if not event.done():
+                        event.set_result(None)
+                    raise tornado.gen.Return(True)
+                event = f.result()
             except TimeoutException:
                 if not minion_running:
                     raise tornado.gen.Return(True)
@@ -1013,13 +1066,13 @@ class SaltAPIHandler(BaseSaltAPIHandler):  # pylint: disable=W0223
                     ping_tag = tagify([ping_pub_data['jid'], 'ret'], 'job')
                     minion_running = False
                     continue
+
             # Minions can return, we want to see if the job is running...
             if event['data'].get('return', {}) == {}:
                 continue
+            if event['data']['id'] not in minions:
+                minions[event['data']['id']] = False
             minion_running = True
-            id_ = event['data']['id']
-            if id_ not in minions_remaining:
-                minions_remaining.append(event['data']['id'])
 
     @tornado.gen.coroutine
     def _disbatch_local_async(self, chunk):
