@@ -17,8 +17,8 @@ Set ``gpg_keydir`` in your config to adjust the homedir the renderer uses.
 
 '''
 
-from __future__ import absolute_import
-import salt.utils
+from __future__ import absolute_import, print_function, unicode_literals
+import salt.loader
 
 
 def ext_pillar(minion_id, pillar, *args, **kwargs):
