@@ -46,11 +46,6 @@ key_accept
 The functions from the proxy minion can be run from the salt commandline using
 the :mod:`salt.modules.nxos<salt.modules.nxos>` execution module.
 
-.. note::
-    The option `proxy_merge_grains_in_module: True` is required to have the NXOS
-    grains be available from the proxy minion, for the 2016.11.0 release.  For
-    Nitrogen, the setting will be True by default.
-
 .. note:
     If `multiprocessing: True` is set for the proxy minion config, each forked
     worker will open up a new connection to the Cisco NX OS Switch.  If you
@@ -58,16 +53,18 @@ the :mod:`salt.modules.nxos<salt.modules.nxos>` execution module.
     `multiprocessing: False`
 
 '''
-from __future__ import absolute_import
+
+# Import Python libs
+from __future__ import absolute_import, print_function, unicode_literals
+import logging
 import multiprocessing
 import re
 
-import salt.utils
+# Import Salt libs
 from salt.utils.pycrypto import gen_hash, secure_password
 from salt.utils.vt_helper import SSHConnection
 from salt.utils.vt import TerminalException
 
-import logging
 log = logging.getLogger(__file__)
 
 __proxyenabled__ = ['nxos']
@@ -81,14 +78,6 @@ def __virtual__():
     '''
     log.info('nxos proxy __virtual__() called...')
 
-    if __opts__.get('proxy_merge_grains_in_module', False) is False:
-        salt.utils.warn_until(
-            'Nitrogen',
-            'To use grains with the NXOS proxy minion, '
-            '`proxy_merge_grains_in_module: True` must be set in the '
-            'proxy minion config.'
-        )
-
     return __virtualname__
 
 
